import java.awt.*;
import java.util.HashMap;
import java.util.Random;

public class DrawingArmoredCar {
    protected EntityArmoredCar armoredCar;

    protected float startPosX;

    protected float startPosY;

    private int pictureWidth;

    private int pictureHeight;

    private int carWidth = 80;

    private int carHeight = 50;

    protected IDrawingCaterpillar drawingCaterpillar;

    public EntityArmoredCar getArmoredCar() {
        return armoredCar;
    }

    public DrawingArmoredCar(int speed, float weight, Color bodyColor) {
        this.armoredCar = new EntityArmoredCar(speed, weight, bodyColor);
        IDrawingCaterpillar[] realisations = new IDrawingCaterpillar[]{
                new DrawingCaterpillar(bodyColor),
                new DrawingCrossCaterpillar(bodyColor),
                new DrawingDoubleCaterpillar(bodyColor)};
        Random r = new Random();
        int num = r.nextInt(4, 7);
        this.drawingCaterpillar = realisations[r.nextInt(3)];
        drawingCaterpillar.setNumRinks(num);
    }

    protected DrawingArmoredCar(int speed, float weight, Color bodyColor, int carWidth, int carHeight) {
        this(speed, weight, bodyColor);
        this.carWidth = carWidth;
        this.carHeight = carHeight;
    }

    public void SetCaterpillar(IDrawingCaterpillar caterpillar) {
        this.drawingCaterpillar = caterpillar;
    }

    public void SetPosition(int x, int y, int width, int height) {
        if (x >= 0 && y >= 0 && x + carWidth < width && y + carHeight < height) {
            startPosX = x;
            startPosY = y;
            pictureWidth = width;
            pictureHeight = height;
        }
    }

    public void MoveTransport(Direction direction) {
        if (pictureWidth < 1|| pictureHeight < 1)
        {
            return;
        }
        switch (direction)
        {
            // вправо
            case Right:
                if (startPosX + carWidth + armoredCar.step < pictureWidth)
                {
                    startPosX += armoredCar.step;
                }
                break;
            //влево
            case Left:
                if (startPosX - armoredCar.step > 0)
                {
                    startPosX -= armoredCar.step;
                }
                break;
            //вверх
            case Up:
                if (startPosY - armoredCar.step > 0)
                {
                    startPosY -= armoredCar.step;
                }
                break;
            //вниз
            case Down:
                if (startPosY + carHeight + armoredCar.step < pictureHeight)
                {
                    startPosY += armoredCar.step;
                }
                break;
        }
    }

    public void DrawTransport(Graphics2D g2d)
    {
        if (startPosX < 0 || startPosY < 0
                || pictureHeight < 1 || pictureWidth < 1)
        {
            return;
        }
        g2d.setPaint(armoredCar.getBodyColor());
        // отрисовка корпуса и гусеницы
        int new_startPosX = Math.round(startPosX);
        int new_startPosY = Math.round(startPosY);
        g2d.fillRect(new_startPosX + 20, new_startPosY, 40, 20);
        g2d.setPaint(Color.LIGHT_GRAY);
        g2d.fillRect(new_startPosX, new_startPosY + 20, 80, 20);

        g2d.fillOval(new_startPosX, new_startPosY + 30, 20, 20);
        g2d.fillOval(new_startPosX + 80 - 20, new_startPosY + 30, 20, 20);
        g2d.fillRect(new_startPosX + 15, new_startPosY + 20, 60, 30);
        // отрисовка катков в гусенице
        drawingCaterpillar.DrawCaterpillar(g2d, new_startPosX, new_startPosY);
    }

    public void ChangeBorders(int width, int height)
    {
        pictureWidth = width;
        pictureHeight = height;
        if (pictureWidth <= carWidth || pictureHeight <= carHeight)
        {
            pictureWidth = 0;
            pictureHeight = 0;
            return;
        }
        if (startPosX + carWidth > pictureWidth)
        {
            startPosX = pictureWidth - carWidth;
        }
        if (startPosY + carHeight > pictureHeight)
        {
            startPosY = pictureHeight - carHeight;
        }
    }

    public HashMap<String, Float> GetCurrentPosition()
    {
        HashMap<String, Float> hashMap = new HashMap<String, Float>();
        hashMap.put("Left", startPosX);
        hashMap.put("Right", startPosX + carWidth);
        hashMap.put("Top", startPosY);
        hashMap.put("Bottom", startPosY + carHeight);
        return hashMap;
    }
}
