import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class FormMapWithArmoredCars extends JFrame{
    private JButton buttonAddArmoredCar;
    private JButton buttonRemoveArmoredCar;
    private JButton buttonShowStorage;
    private JButton buttonShowOnMap;
    private JTextField maskedTextBoxPosition;
    private JComboBox mapSelectorComboBox;
    private JPanel drawPanel;
    private JButton buttonUp;
    private JButton buttonDown;
    private JButton buttonRight;
    private JButton buttonLeft;
    private JPanel mainPanel;
    private JPanel btnPanel;

    private Image img;
    private MapWithSetArmoredCarsGeneric<DrawingObjectArmoredCar, AbstractMap> _mapArmoredCarsCollectionGeneric;

    public FormMapWithArmoredCars() {
        super("Карта с набором объектов");

        setBounds(100, 100, 1000, 700);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        mapSelectorComboBox.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                AbstractMap map = null;
                String name = (String) mapSelectorComboBox.getSelectedItem();
                switch (name)
                {
                    case "Простая карта":
                        map = new SimpleMap();
                        break;
                    case "Карта 1":
                        map = new MyMapWooden();
                        break;
                    case "Карта 2":
                        map = new MyMapLabyrinth();
                        break;
                }
                if (map != null)
                {
                    _mapArmoredCarsCollectionGeneric = new MapWithSetArmoredCarsGeneric<>(
                            drawPanel.getWidth(), drawPanel.getHeight(), map);
                }
                else
                {
                    _mapArmoredCarsCollectionGeneric = null;
                }
            }
        });
        buttonAddArmoredCar.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (_mapArmoredCarsCollectionGeneric == null)
                {
                    return;
                }
                FormArmoredCar form = new FormArmoredCar();
                form.addWindowListener(new WindowAdapter() {
                    @Override
                    public void windowDeactivated(WindowEvent e) {
                        if (form.getSelectedCar() == null)
                            return;
                        DrawingObjectArmoredCar armoredCar = new DrawingObjectArmoredCar(form.getSelectedCar());
                        if (_mapArmoredCarsCollectionGeneric.add(armoredCar) > -1)
                        {
                            if (form.DialogResult) {
                                JOptionPane.showMessageDialog(null, "Объект добавлен");
                                img = _mapArmoredCarsCollectionGeneric.ShowSet();
                                repaint();
                            }
                        }
                        else
                        {
                            JOptionPane.showMessageDialog(null, "Не удалось добавить объект");
                        }
                    }
                });
            }
        });
        buttonRemoveArmoredCar.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (maskedTextBoxPosition.getText().equals(""))
                {
                    return;
                }
                int res = JOptionPane.showConfirmDialog (null,"Удалить объект?",
                        "Удаление", JOptionPane.YES_NO_OPTION);
                if (res == JOptionPane.YES_OPTION) {
                    int pos = Integer.parseInt(maskedTextBoxPosition.getText());
                    if (_mapArmoredCarsCollectionGeneric.remove(pos) != null)
                    {
                        JOptionPane.showMessageDialog(null, "Объект удален");
                        img = _mapArmoredCarsCollectionGeneric.ShowSet();
                    }
                    else
                    {
                        JOptionPane.showMessageDialog(null, "Не удалось удалить объект");
                    }
                }
            }
        });
        buttonShowStorage.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (_mapArmoredCarsCollectionGeneric == null)
                {
                    return;
                }
                img = _mapArmoredCarsCollectionGeneric.ShowSet();
                repaint();
            }
        });

        buttonShowOnMap.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (_mapArmoredCarsCollectionGeneric == null)
                {
                    return;
                }
                img = _mapArmoredCarsCollectionGeneric.ShowOnMap();
                repaint();
            }
        });

        setContentPane(mainPanel);
        setVisible(true);
        buttonLeft.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                img = _mapArmoredCarsCollectionGeneric.MoveObject(Direction.Left);
                repaint();
            }
        });
        buttonUp.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                img = _mapArmoredCarsCollectionGeneric.MoveObject(Direction.Up);
                repaint();
            }
        });
        buttonDown.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                img = _mapArmoredCarsCollectionGeneric.MoveObject(Direction.Down);
                repaint();
            }
        });
        buttonRight.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                img = _mapArmoredCarsCollectionGeneric.MoveObject(Direction.Right);
                repaint();
            }
        });
    }

    private void createUIComponents() {
        drawPanel = new JPanel() {
            @Override
            public void paintComponent(Graphics g) {
                super.paintComponent(g);
                g.drawImage(img, 0, 0, this);
                super.repaint();
            }
        };
    }
}
